/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mex.h"
#include "mp4ff.h"
#include "common.h"
#include "structs.h"
#include "decoder.h"
#include "upsample.h"

uint32_t read_callback(void *user_data, void *buffer, uint32_t length)
{
    return fread(buffer, 1, length, (FILE*)user_data);
}

uint32_t seek_callback(void *user_data, uint64_t position)
{
    return fseek((FILE*)user_data, position, SEEK_SET);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/**********************************/
	/********** Declarations **********/
	/**********************************/

	int k,l;
	char *filename;
	int filename_len;
	FILE *mp4file;
	mp4ff_t *infile;
	int n,numframes;
	unsigned char *buffer;
	unsigned int buffer_size;
	NeAACDecHandle hDecoder;
	NeAACDecFrameInfo info;
	unsigned int samplerate;
	unsigned char channels;
	long buffercount;
	void *sample_buffer;
	double *df;
	int rc;
	long dur;
	int channel,ratio;
	double temp,temp2;
	double save[1024];
	double scale = pow(2,-20.5);
	double sum;

	/**********************************/
	/************* Init ***************/
	/**********************************/

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Open input file */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	mp4file = fopen(filename,"rb");
	if (mp4file==NULL)
		mexErrMsgTxt("Can't open input file\n");

	/* Channel */
	channel = 0;
	if (nrhs>1) channel = (int)mxGetPr(prhs[1])[0];
	if (channel<0||channel>1) mexErrMsgTxt("Channel parameter must be 0 or 1.\n");

	/* Upsample */
	ratio = 2;
	if (nrhs>2) ratio = (int)mxGetPr(prhs[2])[0];
	if (ratio!=1&&ratio!=2&&ratio!=4&&ratio!=8) mexErrMsgTxt("Upsample ratio must be: 1,2,4,8.\n");

	/* Init MPEG-4 decoder */
	mp4ff_callback_t *mp4cb = mxCalloc(1,sizeof(mp4ff_callback_t));
	mp4cb->read = read_callback;
	mp4cb->seek = seek_callback;
	mp4cb->user_data = mp4file;
	infile = mp4ff_open_read(mp4cb);
	if (!infile)
		mexErrMsgTxt("Can't open input file\n");
	if (mp4ff_total_tracks(infile)!=1)
		mexErrMsgTxt("Invalid file\n");

	/* Init AAC decoder */
	buffer = NULL;
	buffer_size = 0;
	mp4ff_get_decoder_config(infile, 0, &buffer, &buffer_size);
	hDecoder = NeAACDecOpen();
	hDecoder->config.outputFormat  = FAAD_FMT_DOUBLE;
	buffercount = NeAACDecInit2(hDecoder, buffer, buffer_size, &samplerate, &channels);
	if (buffer) free(buffer);
	if (channels!=2)
		mexErrMsgTxt("Invalid file\n");
	numframes = mp4ff_num_samples(infile,0);

	/* Init output */
	df = (double*) mxCalloc(numframes,sizeof(double));
	if (nlhs>0) {
		for (k=0;k<1024;k++) save[k] = 0;
		plhs[0] = mxCreateDoubleMatrix(numframes*ratio,1,mxREAL);
	}

	/**********************************/
	/********** Decoding **************/
	/**********************************/

	for (n = 0; n < numframes; n++)	{
		/* Read MPEG-4 frame */
		buffer = NULL;
		buffer_size = 0;
		dur = mp4ff_get_sample_duration(infile, 0, n);
		rc = mp4ff_read_sample(infile, 0, n, &buffer,  &buffer_size);
		/* Decode AAC */
		sample_buffer = NeAACDecDecode(hDecoder, &info, buffer, buffer_size, 8);
		if (buffer) free(buffer);
		if (info.object_type!=2) mexErrMsgTxt("Invalid MPEG-4 object type\n");
		/* DF */
		sum = 0;
		if (hDecoder->fb->window_sequence[0]==2) {
			for (l=0 ; l<8 ; l++) {
				for (k=0 ; k<128 ; k++) {
					temp = (double)hDecoder->fb->spec_coef[channel][l*128+k]*(double)hDecoder->fb->spec_coef[channel][l*128+k]*8;
					temp2 = temp - save[k*8+l];
					save[k*8+l] = temp;
					if (temp2<0) temp2 = -temp2;
					sum += sqrt(temp2);
				}
			}
		} else {
			for (k=0;k<1024;k++) {
				temp = (double)hDecoder->fb->spec_coef[channel][k]*(double)hDecoder->fb->spec_coef[channel][k];
				temp2 = temp - save[k];
				save[k] = temp;
				if (temp2<0) temp2 = -temp2;
				sum += sqrt(temp2);
			}
		}
		/* Output */
		df[n] = sum*scale;
		if (nlhs>0&&ratio==1) mxGetPr(plhs[0])[n] = df[n];
	}

	/* Upsample */
	if (ratio>1) {
		upsample( df, numframes, mxGetPr(plhs[0]), ratio );
		for (n=0;n<numframes*ratio;n++)
			if (mxGetPr(plhs[0])[n]<0)
				mxGetPr(plhs[0])[n]=0;
	}

	/**********************************/
	/*********** Clean up *************/
	/**********************************/

	mxFree( df );
	NeAACDecClose(hDecoder);
	mp4ff_close(infile);
	mxFree(mp4cb);
	fclose(mp4file);
	mxFree(filename);
}

