%AACMDCT Decode AAC and returns MDCT coefficients
%   C = AACMDCT(FILENAME) decodes the given AAC FILENAME and returns
%   MDCT coefficients C of left channel.
%
%   [C,WSE] = AACMDCT(FILENAME) decodes the given AAC FILENAME and returns
%   MDCT coefficients C and window sequence WSE of left channel.
%
%   [C,WSE,WSH] = AACMDCT(FILENAME) decodes the given AAC FILENAME and
%   returns MDCT coefficients C, window sequence WSE and window shape WSH
%   of left channel.
%
%   [...] = AACMDCT(FILENAME,CHANNEL) sets the channel:
%   0: left channel or mono (Default)
%   1: right channel
%
%   [...] = AACMDCT(FILENAME,CHANNEL,STAGE) decodes the given AAC FILENAME
%   until a given STAGE:
%
%   1: after dequantisation and scaling
%   2: after PNS
%   3: after mid/side (same as 2 for mono)
%   4: after intensity stereo (same as 2 for mono)
%   5: after prediction MAIN profile (do nothing for LC)
%   6: after prediction LTP profile (do nothing for LC)
%   7: after TNS
%   8: after DRC (Default)
%   0: after Filterbank
%
%   Output (first channel for stereo):
%      - C: MDCT coefficients
%      - WSE: window sequence (0:LONG,1:LONG_STOP,2:SHORT,3:LONG_START)
%      - WSH: window shape (0:SINE,1:KBD)
%
%   This function decodes MPEG-4 AAC Low Complexity profile only.

%   Mex implementation in aacmdct.c
