clear all
close all

%Parameters
low = 20;
high = 16000;
N = 2048;
M = 40;
Fs = 44100;
filename = 'filters.h';

%Filters parameters
lowestMel = log(1.0+low/700.0)*1127.01048;
highestMel = log(1.0+high/700.0)*1127.01048;
freqs = zeros(1,M+2);
for n=1:M+2
    freqs(n) = 700.0*( exp( (lowestMel+((n-1)/(M+1))*(highestMel-lowestMel)) / 1127.01048 ) - 1 );
end
lower = freqs(1:end-2);
center = freqs(2:end-1);
upper = freqs(3:end);
heights = 2./(upper-lower);

%Filters
fftfs = ((0:N/2-1)+0.5)*44100/N;
filters = zeros(M,N/2);
first = zeros(M,1);
last = zeros(M,1);
for n=1:M
    for m=1:N/2
        if fftfs(m)>lower(n)&&fftfs(m)<=center(n)
            filters(n,m) = heights(n) * (fftfs(m)-lower(n))/(center(n)-lower(n));
        end
        if fftfs(m)>center(n)&&fftfs(m)<=upper(n)
            filters(n,m) = heights(n) * (upper(n)-fftfs(m))/(upper(n)-center(n));
        end
    end
    temp = find(filters(n,:));
    first(n) = temp(1)-1;
    last(n) = temp(end)-1;
end

fid=fopen(filename,'w');

fprintf(fid,'const int filters_first[%d]={',M);
for n=1:length(first)-1
    fprintf(fid,'%d',first(n));
    fprintf(fid,',');
end
fprintf(fid,'%d',first(end));
fprintf(fid,'};\n\n');

fprintf(fid,'const int filters_last[%d]={',M);
for n=1:length(last)-1
    fprintf(fid,'%d',last(n));
    fprintf(fid,',');
end
fprintf(fid,'%d',last(end));
fprintf(fid,'};\n\n');

fprintf(fid,'const double filters_weight[%d]={',M*N/2);
for n=1:M
    for m=1:N/2
        fprintf(fid,'%.20f',filters(n,m));
        if n<M||m<N/2
            fprintf(fid,',');
        end
    end
    fprintf(fid,'\n');
end
fprintf(fid,'};\n\n');
fclose(fid);